using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Media;
using System.Resources;

namespace SocietyGames.SpeechWorkout
{
    public partial class FrameControl : UserControl
    {
        private IExercise exercise;        
        private RankingChangedEventHandler taskCompletedDelegate;
        private RankingChangedEventHandler correctlyAnsweredDelegate;
        private RankingChangedEventHandler incorrectlyAnsweredDelegate;

        private SoundPlayer correctSoundPlayer;
        private SoundPlayer incorrectSoundPlayer;

        private bool hasTimeLeft;
        private readonly int updateInterval = 10000;

        private float ranking;

        public FrameControl()
        {
            InitializeComponent();
            
            //Delegates
            this.taskCompletedDelegate = new RankingChangedEventHandler(exercise_TaskCompleted);
            this.correctlyAnsweredDelegate = new RankingChangedEventHandler(exercise_CorrectlyAnswered);
            this.incorrectlyAnsweredDelegate = new RankingChangedEventHandler(exercise_IncorrectlyAnswered);
            
            //NextButton
            this.nextButton.Top = (this.exercisePanel.Height / 2) - (this.nextButton.Height / 2);
            this.nextButton.Left = (this.exercisePanel.Width / 2) - (this.nextButton.Width / 2);

            //Sounds            
            this.correctSoundPlayer = new SoundPlayer(SocietyGames.SpeechWorkout.Properties.Resources.Correct);
            this.incorrectSoundPlayer = new SoundPlayer(SocietyGames.SpeechWorkout.Properties.Resources.Incorrect);

            //TimerBar
            this.timerBar.TimeEnded += new EventHandler(timerBar_TimeEnded);
        }

        void timerBar_TimeEnded(object sender, EventArgs e)
        {
            this.hasTimeLeft = false;
        }

        public event RankingChangedEventHandler ExerciseCompleted;

        public void StartExercise(IExercise exercise, float ranking, int timeInMilliseconds)
        {
            if (exercise == null)
            {
                throw new ArgumentNullException("exercise");
            }
            if (ranking < 0 || ranking > 1)
            {
                throw new ArgumentOutOfRangeException("ranking");
            }
            SetRanking(ranking);
            this.SuspendLayout();
            if (this.exercise != null)
            {
                this.exercise.TaskCompleted -= this.taskCompletedDelegate;
                this.exercise.CorrectlyAnswered -= this.correctlyAnsweredDelegate;
                this.exercise.IncorrectlyAnswered -= this.incorrectlyAnsweredDelegate;
                this.exercisePanel.Controls.Remove(this.exercise.Control);
            }
            this.exercise = exercise;
            this.exercise.TaskCompleted += this.taskCompletedDelegate;
            this.exercise.CorrectlyAnswered += this.correctlyAnsweredDelegate;
            this.exercise.IncorrectlyAnswered += this.incorrectlyAnsweredDelegate;
            this.exercise.Reset(this.ranking);
            this.exercisePanel.Controls.Add(this.exercise.Control);
            this.exercise.Control.Top = (this.exercisePanel.Height / 2) - (this.exercise.Control.Height / 2);
            this.exercise.Control.Left = (this.exercisePanel.Width / 2) - (this.exercise.Control.Width / 2);

            this.ResumeLayout();
            this.timerBar.Clear();            
            this.nextButton.Visible = true;
            this.hasTimeLeft = true;
            this.timerBar.Start(timeInMilliseconds, this.updateInterval);
        }

        private void SetRanking(float ranking)
        {
            if (ranking < 0 || ranking > 1)
            {
                throw new ArgumentOutOfRangeException("ranking");
            }
            this.ranking = ranking;
            this.countLabel.Text = DisplayScore.CreateDisplayScore(this.ranking);
        }

        void OnExerciseCompleted()
        {
            if (this.ExerciseCompleted != null)
            {
                ExerciseCompleted(this, new RankingChangedEventArgs(this.ranking));
            }
        }

        void exercise_TaskCompleted(object source, RankingChangedEventArgs eventArgs)
        {
            SetRanking(eventArgs.Ranking);            
            if (this.hasTimeLeft)
            {                
                this.nextButton.Visible = true;
            }
            else
            {
                OnExerciseCompleted();
            }
        }

        void exercise_CorrectlyAnswered(object source, RankingChangedEventArgs eventArgs)
        {
            SetRanking(eventArgs.Ranking);            
            this.correctSoundPlayer.Play();            
        }

        void exercise_IncorrectlyAnswered(object source, RankingChangedEventArgs eventArgs)
        {
            SetRanking(eventArgs.Ranking);            
            this.incorrectSoundPlayer.Play();
        }

        private void nextButton_Click(object sender, EventArgs e)
        {
            this.nextButton.Visible = false;
            this.exercise.StartNextTask();
        }
    }
}
