using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using System.Diagnostics;

namespace SocietyGames.SpeechWorkout
{    
    public partial class LauncherControl : UserControl
    {
        private List<IExercise> exercises = new List<IExercise>();
        private List<Button> debugButtons = new List<Button>();
        private TextBox debugRankingTextBox;
        private Record record;
        private int exercisesShown;
        private int nextExerciseIndex;

        public LauncherControl() : this(null)
        {            
        }

        public LauncherControl(Record record)
        {
            InitializeComponent();            
            this.record = record;
            //Create the exercises
            this.exercises.Add(new UpDownControl());
            this.exercises.Add(new WhichOneControl());
            this.exercises.Add(new MatchControl());
            this.exercises.Add(new RepeatControl());

            if (record != null)
            {
                //Fill the record in if it's empty. This will keep everything working correctly on first run.
                foreach (IExercise exercise in this.exercises)
                {
                    if (!this.record.Contains(exercise.Name))
                    {
                        this.record.StartExercise(exercise.Name);
                        this.record.EndExercise(0, TimeSpan.Zero);
                    }
                }
            }

            //DEBUG ONLY!
            DebugCreateUserInterfaceFromExercises();   
        }

        public void PrepareForDisplay()
        {
            if (this.exercisesShown == this.exercises.Count)
            {
                //TODO: Is this really what we want to show when we're done?
                this.doneLabel.Visible = true;
                this.startButton.Visible = false;
            }
            else
            {
                this.nextExerciseIndex = 0;
                for (int index = 0; index < this.exercises.Count; index++)
                {
                    if (this.exercises[index].Name == this.record.LastExerciseRun)
                    {
                        this.nextExerciseIndex = index + 1;
                        if (this.nextExerciseIndex == this.exercises.Count)
                        {
                            this.nextExerciseIndex = 0;
                        }
                        break;
                    }
                }                
            }
        }

        public event LaunchExerciseEventHandler LaunchExercise;
        
        private void OnLaunchExercise(IExercise exercise, float ranking)
        {
            Debug.Assert(exercise != null, "Exercise cannot be null.");
            Debug.Assert(ranking >= 0, "Ranking cannot be less than 0.");

            this.record.StartExercise(exercise.Name);
            if (this.LaunchExercise != null)
            {
                this.LaunchExercise(this, new LaunchExerciseEventArgs(exercise, ranking));
            }
            this.exercisesShown++;
        }

        private void DebugCreateUserInterfaceFromExercises()
        {
            int top = 0;
            for (int on = 0; on < this.exercises.Count; on++)
            {
                Button button = new Button();
                this.debugButtons.Add(button);
                button.Text = ((IExercise)this.exercises[on]).Name;
                this.Controls.Add(button);
                button.Top = top;
                button.Height = 24;
                button.Width = 146;
                button.Click += new EventHandler(debugButton_Click);
                top += button.Height;
            }
            this.debugRankingTextBox = new TextBox();
            this.debugRankingTextBox.Text = "0";
            this.Controls.Add(this.debugRankingTextBox);
            this.debugRankingTextBox.Top = top;
            this.debugRankingTextBox.Height = 15;
            this.debugRankingTextBox.Width = 146;
        }

        void debugButton_Click(object sender, EventArgs e)
        {
            for (int on = 0; on < this.debugButtons.Count; on++)
            {
                if (sender == this.debugButtons[on])
                {
                    float ranking;
                    if (float.TryParse(this.debugRankingTextBox.Text, out ranking) && ranking >= 0 && ranking <= 1)
                    {
                        OnLaunchExercise(this.exercises[on], ranking);
                    }
                    else
                    {
                        MessageBox.Show("Please enter a number from 0 to 1.");
                    }                                        
                }
            }
        }

        private void startButton_Click(object sender, EventArgs e)
        {
            if (this.record.Contains(this.exercises[this.nextExerciseIndex].Name))
            {
                OnLaunchExercise(this.exercises[this.nextExerciseIndex], this.record.GetData(this.exercises[this.nextExerciseIndex].Name).CurrentRanking);
            }
            else
            {
                OnLaunchExercise(this.exercises[this.nextExerciseIndex], 0);
            }
        }
    }

    public class LaunchExerciseEventArgs : EventArgs
    {
        IExercise exercise;
        float ranking;

        public LaunchExerciseEventArgs(IExercise exercise, float ranking)
        {
            if (exercise == null)
            {
                throw new ArgumentNullException("exercise");
            }
            this.exercise = exercise;
            this.ranking = ranking;
        }

        public IExercise Exercise
        {
            get
            {
                return this.exercise;
            }
        }

        public float Ranking
        {
            get
            {
                return this.ranking;
            }
        }
    }

    public delegate void LaunchExerciseEventHandler(object source, LaunchExerciseEventArgs eventArgs);
}
