using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.IO;
using System.Xml;

namespace SocietyGames.SpeechWorkout
{
    public partial class MainForm : Form
    {
        private readonly int minutesPerExercise = 10;
        private ExerciseForm exerciseForm;
        private List<IExercise> exercises = new List<IExercise>();
        private List<ExerciseItemControl> exerciseControls = new List<ExerciseItemControl>();
        private ExerciseStats stats;                

        public MainForm()
        {
            InitializeComponent();
            this.exerciseForm = new ExerciseForm();
            this.exercises.Add(new UpDownControl());
            /*this.exercises.Add(new MatchControl());
            this.exercises.Add(new WhichOneControl());
            this.exercises.Add(new RepeatControl());*/
            CreateUserInterfaceFromExercises();
        }

        private void CreateUserInterfaceFromExercises()
        {
            for (int on = 0; on < this.exercises.Count; on++)
            {            
                ExerciseItemControl control = new ExerciseItemControl();
                control.ExerciseName = ((IExercise)this.exercises[on]).Name;
                this.exerciseControls.Add(control);
                this.exercisesPanel.Controls.Add(control);
                control.Top = control.Height * on;
            }
        }

        private int MinutesToMilliseconds(int minutes)
        {
            return 1000 * 60 * minutes;
        }

        private float GetRanking(IExercise exercise)
        {
            if (exercise == null)
            {
                throw new ArgumentNullException("exercise");
            }
            return this.stats.GetData(exercise.Name).CurrentRanking;
        }

        private string GetStatsDirectory()
        {
            return Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments), "SpeechWorkout");
        }

        private string GetStatsFileName()
        {
            return Path.Combine(GetStatsDirectory(), "data.xml");
        }

        private void TryLoadStats()
        {            
            bool loaded = false;
            string fileName = GetStatsFileName();
            try
            {
                XmlDocument document = new XmlDocument();
                document.Load(fileName);
                this.stats = new ExerciseStats(document.FirstChild);
                loaded = true;
            }
            catch (FileNotFoundException)
            {
                loaded = false;
            }
            catch (DirectoryNotFoundException)
            {
                loaded = false;
            }
            catch (XmlException)
            {
                loaded = false;
            }
            if (!loaded)
            {
                MessageBox.Show("Could not load: " + fileName);
            }
            if (this.stats == null)
            {
                this.stats = new ExerciseStats();
            }
        }

        private void SaveStats()
        {
            if (!Directory.Exists(GetStatsDirectory()))
            {
                try
                {
                    Directory.CreateDirectory(GetStatsDirectory());
                }
                catch (Exception)
                {
                    MessageBox.Show("Could not create directory: " + GetStatsDirectory());
                }
            }
            string fileName = GetStatsFileName();
            try
            {
                XmlTextWriter writer = new XmlTextWriter(fileName, Encoding.Unicode);
                writer.Formatting = Formatting.Indented;
                this.stats.Write(writer);
                writer.Flush();
                writer.Close();
            }
            catch (Exception)
            {
                MessageBox.Show("Could not create file: " + fileName);
            }
        }

        private void FillStatsWithExercises()
        {
            if (this.stats == null)
            {
                throw new InvalidOperationException("Tried to fill null stats.");
            }
            foreach (IExercise exercise in this.exercises)
            {
                if (!this.stats.ContainsData(exercise.Name))
                {
                    this.stats.AddData(new ExerciseData(exercise.Name, TimeSpan.Zero, 0, 0));
                }
            }
        }

        private void SyncUIToStats()
        {
            foreach (ExerciseItemControl control in this.exerciseControls)
            {
                ExerciseData exerciseData = this.stats.GetData(control.ExerciseName);
                control.Time = exerciseData.Time;
                control.InitialScore = exerciseData.InitialRanking;
                control.CurrentScore = exerciseData.CurrentRanking;                
            }
        }

        private void RunExercise(IExercise exercise)
        {
            if (exercise == null)
            {
                throw new ArgumentNullException("exercise");
            }
            this.exerciseForm.ShowDialog(this, exercise, GetRanking(exercise), MinutesToMilliseconds(this.minutesPerExercise));
            ExerciseData exerciseData = this.stats.GetData(exercise.Name);
            exerciseData.Time += this.exerciseForm.Time; //new TimeSpan(0, 0, 0, 0, MinutesToMilliseconds(this.minutesPerExercise));
            if (exerciseData.InitialRanking == 0)
            {
                exerciseData.CurrentRanking = exercise.Ranking;
            }
            exerciseData.CurrentRanking = exercise.Ranking;
            SyncUIToStats();
        }

        private void MainForm_Load(object sender, EventArgs e)
        {
            TryLoadStats();
            FillStatsWithExercises();
            SyncUIToStats();
        }

        private void MainForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            SaveStats();
        }

        private void nextButton_Click(object sender, EventArgs e)
        {
            //Choose the exercise with the lowest time.
            TimeSpan lowestTime = TimeSpan.MaxValue;
            IExercise nextExercise = null;
            foreach (IExercise exercise in this.exercises)
            {
                ExerciseData data = this.stats.GetData(exercise.Name);
                if (data.Time < lowestTime)
                {
                    lowestTime = data.Time;
                    nextExercise = exercise;
                }
            }
            //DEBUG!
            //nextExercise = this.exercises[3];
            //END DEBUG!
            RunExercise(nextExercise);
        }
    }
}