using System;
using System.Collections.Generic;
using System.Text;

namespace SocietyGames.SpeechWorkout
{
    class RangeSineWaveOscillator : IOscillator
    {
        private float startFrequency;
        private float endFrequency;
        private int sampleValue;        
        private int sampleRate;
        private float amplitude;
        private int bytesPerSample;

        public RangeSineWaveOscillator(float startFrequency, float endFrequency, int sampleRate, float amplitude, int bytesPerSample)
        {
            if (startFrequency <= 0)
            {
                throw new ArgumentOutOfRangeException("startFrequency");
            }
            if (endFrequency <= 0)
            {
                throw new ArgumentOutOfRangeException("endFrequency");
            }
            if (sampleRate <= 0)
            {
                throw new ArgumentOutOfRangeException("sampleRate");
            }
            if (amplitude < 0)
            {
                throw new ArgumentOutOfRangeException("amplitude");
            }
            if (bytesPerSample <= 0)
            {
                throw new ArgumentOutOfRangeException("bytesPerSample");
            }

            this.startFrequency = startFrequency;
            this.endFrequency = endFrequency;
            this.sampleRate = sampleRate;
            this.amplitude = amplitude;
            this.sampleValue = (int)((double)amplitude * 32767.0);
            this.bytesPerSample = bytesPerSample;
        }

        private void FillLinear(double total, double start, double end, out int totalNumber, out double delta)
        {
            if (total < start + end)
            {
                throw new ArgumentOutOfRangeException("total");
            }
            if (start < 0)
            {
                throw new ArgumentOutOfRangeException("start");
            }
            if (end < 0)
            {
                throw new ArgumentOutOfRangeException("end");
            }

            double nTemp = ((total - (start + end)) / (start + end)) / 0.5;
            int n = (int)(nTemp);
            totalNumber = n + 2;
            delta = (end - start) / ((double)(n) + 1);
        }

        #region IOscillator Members

        public void Generate(byte[] data, int samples)
        {
            if (data == null)
            {
                throw new ArgumentNullException("data");
            }
            if (samples < 0)
            {
                throw new ArgumentOutOfRangeException("samples");
            }
            if (data.Length < samples * this.bytesPerSample)
            {
                throw new ArgumentException("data");
            }
            
            int numberOfWaves;
            double delta;
            FillLinear(samples, ((double)this.sampleRate) / this.startFrequency, ((double)this.sampleRate) / this.endFrequency, out numberOfWaves, out delta);
            int onData = 0;
            double currentFrequency = this.startFrequency;
            double frequencyDelta = (this.endFrequency - this.startFrequency) / ((double)(numberOfWaves) + 1);
            for (int onWave = 0; onWave < numberOfWaves; onWave++)
            {
                double dK = currentFrequency * (Math.PI * 2) / (double)this.sampleRate;
                int samplesThisWave = (int)(((double)sampleRate) / currentFrequency);
                for (int j = 0; j < samplesThisWave; j++)
                {
                    double dSinVal = Math.Cos((double)j * dK);
                    double dAmpVal = 32767.0 * dSinVal * (double)this.amplitude;
                    int nAmpVal = (int)(dAmpVal);
                    for (int index = 0; index < this.bytesPerSample; index++)
                    {
                        data[onData] = (byte)(nAmpVal >> (index * 8));
                        onData++;
                    }
                }
                currentFrequency += frequencyDelta;
            }
            for (; onData < samples; onData++)
            {
                data[onData] = 0;
            }
        }

        #endregion
    }
}
