using System;
using System.Collections.Generic;
using System.Text;

namespace SocietyGames.SpeechWorkout
{
    class SineWaveOscillator
    {
        /// <summary>
        /// 
        /// </summary>
        /// <param name="buffer">The data buffer to write to.</param>
        /// <param name="startOffset">The starting position to write to in the buffer (in bytes, not samples)</param>
        /// <param name="sampleCount">The number of samples to fill in.</param>
        /// <param name="startFrequency">The frequency to begin at.</param>
        /// <param name="endFrequency">The frequency to end at.</param>
        /// <param name="sampleRate">The number of samples per second.</param>
        /// <param name="amplitude"></param>
        /// <param name="bytesPerSample">The number of bytes per sample.</param>
        /// <returns>The next free space that can written to.</returns>
        public static int Generate(byte[] buffer, int startOffset, int sampleCount, float startFrequency, float endFrequency, int sampleRate, float amplitude, int bytesPerSample)
        {
            if (buffer == null)
            {
                throw new ArgumentNullException("buffer");
            }
            if (startOffset < 0 || startOffset >= buffer.Length)
            {
                throw new ArgumentOutOfRangeException("startOffset");
            }
            if (sampleCount < 0 || startOffset + (sampleCount * bytesPerSample) > buffer.Length)
            {
                throw new ArgumentOutOfRangeException("byteCount");
            }
            if (startFrequency < 0)
            {
                throw new ArgumentOutOfRangeException("startFrequency");
            }
            if (endFrequency < 0)
            {
                throw new ArgumentOutOfRangeException("endFrequency");
            }
            if (sampleRate <= 0)
            {
                throw new ArgumentOutOfRangeException("sampleRate");
            }
            if (amplitude < 0)
            {
                throw new ArgumentOutOfRangeException("amplitude");
            }
            if (bytesPerSample <= 0)
            {
                throw new ArgumentOutOfRangeException("bytesPerSample");
            }

            int numberOfWaves;
            double delta;
            FillLinear(sampleCount, ((double)sampleRate) / startFrequency, ((double)sampleRate) / endFrequency, out numberOfWaves, out delta);
            int onData = startOffset;
            double currentFrequency = startFrequency;
            double frequencyDelta = (endFrequency - startFrequency) / ((double)(numberOfWaves) + 1);
            for (int onWave = 0; onWave < numberOfWaves; onWave++)
            {
                double dK = currentFrequency * (Math.PI * 2) / (double)sampleRate;
                int samplesThisWave = (int)(((double)sampleRate) / currentFrequency);
                for (int j = 0; j < samplesThisWave; j++)
                {
                    double dSinVal = Math.Cos((double)j * dK);
                    double dAmpVal = 32767.0 * dSinVal * (double)amplitude;
                    int nAmpVal = (int)(dAmpVal);
                    for (int index = 0; index < bytesPerSample; index++)
                    {
                        buffer[onData] = (byte)(nAmpVal >> (index * 8));
                        onData++;
                    }
                }
                currentFrequency += frequencyDelta;
            }
            return onData;
        }

        private static void FillLinear(double total, double start, double end, out int totalNumber, out double delta)
        {
            if (total < start + end)
            {
                throw new ArgumentOutOfRangeException("total");
            }
            if (start < 0)
            {
                throw new ArgumentOutOfRangeException("start");
            }
            if (end < 0)
            {
                throw new ArgumentOutOfRangeException("end");
            }

            double nTemp = ((total - (start + end)) / (start + end)) / 0.5;
            int n = (int)(nTemp);
            totalNumber = n + 2;
            delta = (end - start) / ((double)(n) + 1);
        }
    }
}
