using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Text;
using System.Windows.Forms;
using Microsoft.DirectX;
using Microsoft.DirectX.DirectSound;

//Reset() Reset everything that is exercise wide.
//StartNewTask() Clear everything that's task wide, play sound, enable buttons.
//On click finish -- complete task.

namespace SocietyGames.SpeechWorkout
{
    public partial class UpDownControl : ExerciseControl, IExercise
    {
        enum ButtonType
        {
            None,
            Up,
            Down
        }

        #region Settings
        readonly float maxPause = 0.25f; //500 ms
        readonly float minPause = 0.01f; //10 ms
        readonly float maxPlayLength = 0.25f; //250 ms
        readonly float minPlayLength = 0.06f; //60 ms
        readonly int lowFrequency = 1000;
        #endregion

        #region Specific Variables
        Device device;        
        List<ButtonType> pressedButtons = new List<ButtonType>();
        List<ButtonType> pattern = new List<ButtonType>();
        float currentRanking;
        UpDownSound sound;
        Staircase staircase = new Staircase(3, 1);
        readonly float rankingAdjustmentRate = 0.01f;
        Random random = new Random();
        #endregion

        public UpDownControl()
        {            
            InitializeComponent();            
            this.device = new Device();            
            this.device.SetCooperativeLevel(this, CooperativeLevel.Priority);            
        }

        private void ProcessAnswer()
        {
            if (this.pattern.Count != this.pressedButtons.Count)
            {
                throw new InvalidOperationException();
            }

            //Check if it's correct
            bool isCorrect = true;
            for (int onCheck = 0; onCheck < this.pattern.Count; onCheck++)
            {
                if (this.pattern[onCheck] != this.pressedButtons[onCheck])
                {
                    isCorrect = false;
                }
            }
            int delta = this.staircase.Score(isCorrect);
            this.currentRanking += delta * this.rankingAdjustmentRate;            
            if (this.currentRanking < 0)
            {
                this.currentRanking = 0;
            }
            if (this.currentRanking > 1)
            {
                this.currentRanking = 1;
            }
            if (isCorrect)
            {
                OnCorrectlyAnswered(this.currentRanking);                
            }
            else
            {
                OnIncorrectlyAnswered(this.currentRanking);       
            }
            this.upButton.Visible = false;
            this.downButton.Visible = false;
            OnTaskCompleted(this.currentRanking);
        }

        private void PlaySound()
        {   
            if (this.sound != null) //Kill any old sounds.
            {
                this.sound.Stop();
                this.sound.Dispose();
            }                
            this.pattern.Clear();
            for (int on = 0; on < 2; on++)
            {
                if (this.random.NextDouble() < 0.5)
                {
                    this.pattern.Add(ButtonType.Up);
                }
                else
                {
                    this.pattern.Add(ButtonType.Down);
                }
            }
            GenerateSoundFromRankingAndPattern();
            //Update our internal data
            this.pressedButtons.Clear();
            //Update the UI
            //this.upButton.Visible = false;
            //this.downButton.Visible = false;
            //
            this.sound.Play(0, BufferPlayFlags.Default);
        }

        private void GenerateSoundFromRankingAndPattern()
        {            
            float pause = minPause + (maxPause - minPause) * (1 - this.currentRanking);            
            float playLength = minPlayLength + (maxPlayLength - minPlayLength) * (1 - this.currentRanking);            
            int highFrequency;
            UpDownParameters parameters;
            if (playLength <= 0.08f)
            {
                highFrequency = (int)(65536 * playLength);
            }
            else
            {
                highFrequency = 2428;
            }
            int frequency0;
            int frequency1;
            int frequency2;
            int frequency3;
            if (this.pattern[0] == ButtonType.Up)
            {
                frequency0 = lowFrequency;
                frequency1 = highFrequency;
            }
            else
            {
                frequency0 = highFrequency;
                frequency1 = lowFrequency;
            }
            if (this.pattern[1] == ButtonType.Up)
            {
                frequency2 = lowFrequency;
                frequency3 = highFrequency;
            }
            else
            {
                frequency2 = highFrequency;
                frequency3 = lowFrequency;
            }
            parameters = new UpDownParameters(frequency0, frequency1, playLength, pause, frequency2, frequency3, playLength);
            this.sound = new UpDownSound(this.device, parameters);
        }

        #region IExercise Members

        string IExercise.Name
        {
            get
            {
                return "Up or Down";
            }
        }

        public /*override*/ new void Reset(float ranking)
        {
            if (ranking < 0 || ranking > 1)
            {
                throw new ArgumentOutOfRangeException("ranking");
            }

            this.currentRanking = ranking;
            this.staircase.Clear();
            this.upButton.Visible = false;
            this.downButton.Visible = false;
        }

        public /*override*/ new void StartNextTask()
        {
            PlaySound();
            this.upButton.Visible = true;
            this.downButton.Visible = true;
        }

        #endregion

        private void upButton_Click(object sender, EventArgs e)
        {
            this.pressedButtons.Add(ButtonType.Up);
            //TODO: Figure out the number based on the number of sounds.
            if (this.pressedButtons.Count == this.pattern.Count)
            {
                ProcessAnswer();
            }
        }

        private void downButton_Click(object sender, EventArgs e)
        {
            this.pressedButtons.Add(ButtonType.Down);
            //TODO: Figure out the number based on the number of sounds.
            if (this.pressedButtons.Count == this.pattern.Count)
            {
                ProcessAnswer();
            }
        }
    }
}
