using System;
using System.Collections.Generic;
using System.Text;
using Microsoft.DirectX;
using Microsoft.DirectX.DirectSound;

namespace SocietyGames.SpeechWorkout
{
    class UpDownParameters
    {
        private int firstStart;
        private int firstEnd;
        private float firstLength;
        private float delayLength;
        private int secondStart;
        private int secondEnd;
        private float secondLength;

        public UpDownParameters(int firstStart, int firstEnd, float firstLength,
                                float delayLength,
                                int secondStart, int secondEnd, float secondLength)
        {
            if (firstStart <= 0)
            {
                throw new ArgumentOutOfRangeException("firstStart");
            }
            if (firstEnd <= 0)
            {
                throw new ArgumentOutOfRangeException("firstEnd");
            }
            if (firstLength < 0)
            {
                throw new ArgumentOutOfRangeException("firstLength");
            }
            if (delayLength < 0)
            {
                throw new ArgumentOutOfRangeException("delayLength");
            }
            if (secondStart <= 0)
            {
                throw new ArgumentOutOfRangeException("secondStart");
            }
            if (secondEnd <= 0)
            {
                throw new ArgumentOutOfRangeException("secondEnd");
            }
            if (secondLength < 0)
            {
                throw new ArgumentOutOfRangeException("secondLength");
            }
            this.firstStart = firstStart;
            this.firstEnd = firstEnd;
            this.firstLength = firstLength;
            this.delayLength = delayLength;
            this.secondStart = secondStart;
            this.secondEnd = secondEnd;
            this.secondLength = secondLength;
        }

        public int FirstStart
        {
            get
            {
               return this.firstStart;
            }
        }
        public int FirstEnd
        {
            get
            {
                return this.firstEnd;
            }
        }
        
        public float FirstLength
        {
            get
            {
                return this.firstLength;
            }
        }
        
        public float DelayLength
        {
            get
            {
                return this.delayLength;
            }
        }
        public int SecondStart
        {
            get
            {
                return this.secondStart;
            }
        }
        public int SecondEnd
        {
            get
            {
                return this.secondEnd;
            }
        }
        
        public float SecondLength
        {
            get
            {
                return this.secondLength;   
            }
        }
    }

    class UpDownSound : IDisposable
    {
        UpDownParameters parameters;
        SecondaryBuffer buffer;
        bool isDisposed = false;

        public UpDownSound(Device device, UpDownParameters parameters)
        {
            if (device == null)
            {
                throw new ArgumentNullException("device");
            }
            if (parameters == null)
            {
                throw new ArgumentNullException("parameters");
            }
            
            this.parameters = parameters;            
            
            //Set the wave format
            WaveFormat waveFormat = new WaveFormat();
            waveFormat.SamplesPerSecond = 44100;
            waveFormat.Channels = 1;
            waveFormat.BitsPerSample = 16;
            waveFormat.AverageBytesPerSecond = waveFormat.SamplesPerSecond * waveFormat.Channels * (waveFormat.BitsPerSample / 8);
            waveFormat.BlockAlign = (short)((waveFormat.BitsPerSample / 8) * waveFormat.Channels);
            waveFormat.FormatTag = WaveFormatTag.Pcm;

            int firstSampleCount = (int)Math.Ceiling(this.parameters.FirstLength * waveFormat.SamplesPerSecond);
            int delaySampleCount = (int)Math.Ceiling(this.parameters.DelayLength * waveFormat.SamplesPerSecond);
            int secondSampleCount = (int)Math.Ceiling(this.parameters.SecondLength * waveFormat.SamplesPerSecond);

            float seconds = this.parameters.FirstLength + this.parameters.DelayLength + this.parameters.SecondLength;            
            BufferDescription bufferDescription = new BufferDescription();
            bufferDescription.Flags = BufferDescriptionFlags.LocateInSoftware | BufferDescriptionFlags.GlobalFocus | BufferDescriptionFlags.ControlVolume;
            bufferDescription.BufferBytes = (firstSampleCount + delaySampleCount + secondSampleCount) * waveFormat.BlockAlign;
            bufferDescription.Format = waveFormat;
            this.buffer = new SecondaryBuffer(bufferDescription, device);

            byte[] data = new byte[bufferDescription.BufferBytes];
            int offset = 0;
            offset = SineWaveOscillator.Generate(data, offset, firstSampleCount, parameters.FirstStart, parameters.FirstEnd, waveFormat.SamplesPerSecond, 0.25f, waveFormat.BlockAlign);
            //Fill the delay with nothing.            
            for (int onSample = 0; onSample < delaySampleCount; onSample++)
            {
                for (int onByte = 0; onByte < waveFormat.BlockAlign; onByte++)
                {
                    data[offset] = 0;
                    offset++;
                }
            }
            offset = SineWaveOscillator.Generate(data, offset, secondSampleCount, parameters.SecondStart, parameters.SecondEnd, waveFormat.SamplesPerSecond, 0.25f, waveFormat.BlockAlign);                
            this.buffer.Write(0, data, LockFlag.None);
            this.buffer.Volume = -1000;            
        }

        public void Play(int priority, BufferPlayFlags flags)
        {
            if (this.isDisposed)
            {
                throw new ObjectDisposedException("Sound");
            }            
            this.buffer.Play(priority, flags);
        }

        public void Stop()
        {
            this.buffer.Stop();
        }

        public UpDownParameters UpDownParameters
        {
            get
            {
                return this.parameters;
            }
        }

        private void Generate(UpDownParameters parameters, byte[] data, int samples)
        {
            if (parameters == null)
            {
                throw new ArgumentNullException("parameters");
            }
            if (data == null)
            {
                throw new ArgumentNullException("data");
            }
            if (samples < 0)
            {
                throw new ArgumentOutOfRangeException("samples");
            }

        }

        #region IDisposable Members

        ~UpDownSound()
        {
            Dispose(false);
        }

        protected virtual void Dispose(bool isDisposing)
        {
            if (this.isDisposed)
            {
                return;
            }
            if (isDisposing)
            {
                //Free managed resources
                if (this.buffer != null)
                {
                    this.buffer.Dispose();
                }
            }
            //Free unmanaged
            this.isDisposed = true;
        }

        public void Dispose()
        {
            Dispose(true);
            GC.SuppressFinalize(this);
        }

        #endregion
    }
}
